/*-
 * Copyright (c) 2009 Pietro Cerutti <gahr@gahr.ch>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>

#include "chans.h"
#include "callbacks.h"
#include "gui_helper.h"

static Ewl_Container *container;
static Ewl_Widget **chanBoxes;

void
create_chans(Ewl_Container *c)
{
	container = c;
	reload_chans();
}

void
reload_chans(void)
{
	/* flush container */
	Ewl_Widget *child;
	Ewl_Widget *box;
	int nof_boxes = ewl_container_child_count_get(container);
	int nof_children;
	while (nof_boxes--) {
		box = ewl_container_child_get(EWL_CONTAINER(container), nof_boxes);
		nof_children = ewl_container_child_count_get(box);
		while (nof_children--) {
			child = ewl_container_child_get(EWL_CONTAINER(box), nof_children);
			ewl_container_child_remove(EWL_CONTAINER(box), child);
			ewl_widget_free(child);
			printf("Removing child\n");
		}
		ewl_container_child_remove(EWL_CONTAINER(container), box);
		ewl_widget_free(box);
		printf("Removing box\n");
	}
	free(chanBoxes);

	/* add channels */
	int i;
	int nof_chans = mixer_get_nof_chans();
	const chan_t *chans = mixer_get_chans();

	chanBoxes = malloc(nof_chans * sizeof(Ewl_Container *));
	ewl_object_minimum_size_set(EWL_OBJECT(container), nof_chans * CHAN_WIDTH, 0);

	for (i=0; i<nof_chans; i++) {

		/* container */
		Ewl_Widget *chanBox = ewl_border_new();
		ewl_border_label_set(EWL_BORDER(chanBox), mixer_get_chan_name(chans[i]));
		ewl_container_child_append(EWL_CONTAINER(container), chanBox);
		ewl_widget_show(chanBox);
		chanBoxes[i] = chanBox;

		/* seeker */
		Ewl_Widget *seeker = ewl_seeker_new();
		ewl_container_child_append(EWL_CONTAINER(chanBox), seeker);
		ewl_seeker_orientation_set(EWL_SEEKER(seeker), EWL_ORIENTATION_VERTICAL);
		ewl_object_fill_policy_set(EWL_OBJECT(seeker), EWL_FLAG_FILL_VFILL | EWL_FLAG_FILL_VSHRINK);
		ewl_object_minimum_size_set(EWL_OBJECT(seeker), 20, CHAN_HEIGHT - 30);
		ewl_object_maximum_size_set(EWL_OBJECT(seeker), 20, CHAN_HEIGHT - 30);
		//ewl_object_size_request(EWL_OBJECT(seeker), 20, CHAN_HEIGHT - 30);
		ewl_range_minimum_value_set(EWL_RANGE(seeker), 0.);
		ewl_range_maximum_value_set(EWL_RANGE(seeker), 100.);
		ewl_range_step_set(EWL_RANGE(seeker), 1);
		ewl_range_invert_set(EWL_RANGE(seeker), 1);
		ewl_callback_append(seeker, EWL_CALLBACK_VALUE_CHANGED, gain_seeker_value_change, (void *)chans[i]);
		ewl_callback_append(seeker, EWL_CALLBACK_MOUSE_WHEEL, gain_seeker_mouse_wheel, (void *)chans[i]);
		ewl_widget_show(seeker);

		/* gain label */
		Ewl_Widget *label = ewl_text_new();
		ewl_container_child_append(EWL_CONTAINER(chanBox), label);
		ewl_object_alignment_set(EWL_OBJECT(label), EWL_FLAG_ALIGN_CENTER);
		ewl_object_fill_policy_set(EWL_OBJECT(label), EWL_FLAG_FILL_HFILL | EWL_FLAG_FILL_HSHRINK);
		ewl_widget_show(label);
	}

	epromix_win_best_size();
}

int
update_chans(void *_unused)
{
	int i, l, r, g;
	int nof_chans = mixer_get_nof_chans();
	const chan_t *chans = mixer_get_chans();
	char buf[6];

	for (i=0; i<nof_chans; i++) {
		Ewl_Widget *c = chanBoxes[i];
		mixer_get_chan_gain(chans[i], &l, &r);
		g = (int)(0.5 * (l + r));
		snprintf(buf, sizeof(buf)-1, "%3d %%", g);
		Ewl_Range *r = EWL_RANGE(ewl_container_child_get(EWL_CONTAINER(c), 0));
		Ewl_Text  *t = EWL_TEXT(ewl_container_child_get(EWL_CONTAINER(c), 1));
		ewl_range_value_set(r, g);
		epromix_set_colored_text(t, buf, 0, 0, 0, 255);
	}
	return (1);
}
