/*-
 * Copyright (c) 2009 Pietro Cerutti <gahr@gahr.ch>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *	notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *	notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "mixer.h"
#include <limits.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>

static const char *device_labels[] = SOUND_DEVICE_LABELS;
static const char *device_names[]  = SOUND_DEVICE_NAMES;

struct mixer_ {
	char mixer_dev[PATH_MAX];
	int *chans;
	int nrchans;
	int mixer_fd;
	int devmask;
	int recmask;
};

static mixer_t mixer_inst = NULL;

mixer_t
mixer_get_inst(void);

int
mixer_init(char *mixer_dev)
{
	mixer_t m;
	int i, j;
	int fd = -1;
	int devmask = 0;
	int recmask = 0;

	mixer_inst = NULL;

	if (!mixer_dev) {
		return (-1);
	}

	if ((fd = open(mixer_dev, O_RDONLY)) == -1) {
		return (-1);
	}

	if (ioctl(fd, SOUND_MIXER_READ_DEVMASK, &devmask) == -1) {
		return (-1);
	}

	if (ioctl(fd, SOUND_MIXER_READ_RECMASK, &recmask) == 1) {
		return (-1);
	}

	if (!(m = malloc(sizeof (struct mixer_)))) {
		close(fd);
		return (-1);
	}

	strncpy(m->mixer_dev, mixer_dev, sizeof (m->mixer_dev));
	m->mixer_fd = fd;
	m->devmask = devmask;
	m->recmask = recmask;
	m->nrchans = 0;

	for (i=0; i<SOUND_MIXER_NRDEVICES; i++) {
		if ( (1<<i) & m->devmask) {
			m->nrchans++;
		}
	}
	if (!(m->chans = malloc(sizeof(int) * m->nrchans))) {
		mixer_destroy();
		return (-1);
	}
	for (i=0, j=0; i<SOUND_MIXER_NRDEVICES; i++) {
		if ( (1<<i) & m->devmask) {
			m->chans[j++] = i;
		}
	}

	mixer_inst = m;
	return (0);
}

void
mixer_destroy(void)
{
	if (mixer_inst) {
		close(mixer_inst->mixer_fd);
		free(mixer_inst->chans);
	}
	free(mixer_inst);
}

int
mixer_get_nof_chans(void)
{
	return (mixer_inst->nrchans);
}

const chan_t *
mixer_get_chans()
{
	return (mixer_inst->chans);
}

const char *
mixer_get_chan_name(const chan_t c)
{
	return device_names[c];
}

int
mixer_get_chan_gain(chan_t c, int *left, int *right)
{
	int gain = 0;
	if (ioctl(mixer_inst->mixer_fd, MIXER_READ(c), &gain) == -1) {
		return (-1);
	}

	if (left) {
		*left = gain & 0xff;
	}

	if (right) {
		*right = gain >>8 & 0xff;
	}

	return (0);
}

int
mixer_set_chan_gain(chan_t c, int left, int right)
{
	int gain = left | right << 8;

	if (ioctl(mixer_inst->mixer_fd, MIXER_WRITE(c), &gain) == -1) {
		return (1);
	}

	return (gain == -1) ? 1 : 0;
}

int
mixer_can_record(chan_t c)
{
	return (mixer_inst->recmask & (1<<c));
}

int
mixer_is_recording(chan_t c)
{
	int recsrc;

	if (ioctl(mixer_inst->mixer_fd, SOUND_MIXER_READ_RECSRC, &recsrc) == -1) {
		return (1);
	}

	return (recsrc & (1<<c));
}

int
mixer_set_recording(chan_t c)
{
	if (!mixer_can_record(c)) {
		return (-1);
	}

	int recsrc = 1<<c;

	if (ioctl(mixer_inst->mixer_fd, SOUND_MIXER_WRITE_RECSRC, &recsrc) == -1) {
		return (-1);
	}

	return (0);
}

int
mixer_add_recording(chan_t c)
{
	return (mixer_is_recording(c) ? 0 : mixer_toggle_recording(c));
}

int
mixer_rem_recording(chan_t c)
{
	return (mixer_is_recording(c) ? mixer_toggle_recording(c) : 0);
}

int
mixer_toggle_recording(chan_t c)
{
	int recsrc = 0;

	if (ioctl(mixer_inst->mixer_fd, SOUND_MIXER_READ_RECSRC, &recsrc) == -1) {
		return (1);
	}

	recsrc ^= 1<<c;

	if (ioctl(mixer_inst->mixer_fd, SOUND_MIXER_WRITE_RECSRC, &recsrc) == -1) {
		return (-1);
	}

	return (0);
}
